From 7b7db873ac9520b31000f0662ca389779a355aa5 Mon Sep 17 00:00:00 2001
From: Ajin Cherian <ajinc@fast.au.fujitsu.com>
Date: Fri, 27 Aug 2021 06:21:00 -0400
Subject: [PATCH v2] fix for tap test failure seen in 001_rep_changes

When a walsender restarts, the new walsender might start before the previous walsender
relinquishes the slot. This will cause the new walsender to fail to acquire the slot
and fail immediately. This can cause the script to wrongly assume that the new walsender
has started. Then when the script tries to get the pid of the walsender and gets a NULL
, further queries will fail with an error and the test fails. This fix adds a check
to confirm that the new walsender reaches a 'streaming' state before continuing.
---
 src/test/subscription/t/001_rep_changes.pl      | 6 +++---
 src/test/subscription/t/022_twophase_cascade.pl | 8 ++++----
 2 files changed, 7 insertions(+), 7 deletions(-)

diff --git a/src/test/subscription/t/001_rep_changes.pl b/src/test/subscription/t/001_rep_changes.pl
index 0c84d87..03cf3e3 100644
--- a/src/test/subscription/t/001_rep_changes.pl
+++ b/src/test/subscription/t/001_rep_changes.pl
@@ -424,7 +424,7 @@ $node_subscriber->safe_psql('postgres',
 	"ALTER SUBSCRIPTION tap_sub CONNECTION '$publisher_connstr sslmode=disable'"
 );
 $node_publisher->poll_query_until('postgres',
-	"SELECT pid != $oldpid FROM pg_stat_replication WHERE application_name = 'tap_sub';"
+	"SELECT pid != $oldpid FROM pg_stat_replication WHERE application_name = 'tap_sub' AND state = 'streaming';"
 ) or die "Timed out while waiting for apply to restart";
 
 $oldpid = $node_publisher->safe_psql('postgres',
@@ -434,7 +434,7 @@ $node_subscriber->safe_psql('postgres',
 	"ALTER SUBSCRIPTION tap_sub SET PUBLICATION tap_pub_ins_only WITH (copy_data = false)"
 );
 $node_publisher->poll_query_until('postgres',
-	"SELECT pid != $oldpid FROM pg_stat_replication WHERE application_name = 'tap_sub';"
+	"SELECT pid != $oldpid FROM pg_stat_replication WHERE application_name = 'tap_sub' AND state = 'streaming';"
 ) or die "Timed out while waiting for apply to restart";
 
 $node_publisher->safe_psql('postgres',
@@ -488,7 +488,7 @@ $oldpid = $node_publisher->safe_psql('postgres',
 $node_subscriber->safe_psql('postgres',
 	"ALTER SUBSCRIPTION tap_sub RENAME TO tap_sub_renamed");
 $node_publisher->poll_query_until('postgres',
-	"SELECT pid != $oldpid FROM pg_stat_replication WHERE application_name = 'tap_sub_renamed';"
+	"SELECT pid != $oldpid FROM pg_stat_replication WHERE application_name = 'tap_sub_renamed' AND state = 'streaming';"
 ) or die "Timed out while waiting for apply to restart";
 
 # check all the cleanup
diff --git a/src/test/subscription/t/022_twophase_cascade.pl b/src/test/subscription/t/022_twophase_cascade.pl
index adda874..a38c5e7 100644
--- a/src/test/subscription/t/022_twophase_cascade.pl
+++ b/src/test/subscription/t/022_twophase_cascade.pl
@@ -235,10 +235,10 @@ is($result, qq(21), 'Rows committed are present on subscriber C');
 
 my $oldpid_B = $node_A->safe_psql('postgres', "
 	SELECT pid FROM pg_stat_replication
-	WHERE application_name = '$appname_B';");
+	WHERE application_name = '$appname_B' AND state = 'streaming';");
 my $oldpid_C = $node_B->safe_psql('postgres', "
 	SELECT pid FROM pg_stat_replication
-	WHERE application_name = '$appname_C';");
+	WHERE application_name = '$appname_C' AND state = 'streaming';");
 
 # Setup logical replication (streaming = on)
 
@@ -253,11 +253,11 @@ $node_C->safe_psql('postgres', "
 
 $node_A->poll_query_until('postgres', "
 	SELECT pid != $oldpid_B FROM pg_stat_replication
-	WHERE application_name = '$appname_B';"
+	WHERE application_name = '$appname_B' AND state = 'streaming';"
 ) or die "Timed out while waiting for apply to restart";
 $node_B->poll_query_until('postgres', "
 	SELECT pid != $oldpid_C FROM pg_stat_replication
-	WHERE application_name = '$appname_C';"
+	WHERE application_name = '$appname_C' AND state = 'streaming';"
 ) or die "Timed out while waiting for apply to restart";
 
 ###############################
-- 
1.8.3.1

